/**
 * TODO: 
 * email debugging info
 * Subnet viewer
 * Easy "Scan My Subnet" button
 * Show IP address, if connected to wifi
 * Licensing
 * Show notification when completed in notification area.
 * Disable scan button while scanning (synchronization).
 * Long-press on PipsViewResultActivity should OPEN, SHARE, DELETE.
 * Add OnOptionsMenu to each activity.
 *  
 * (gif): save scan results to db in Scan class, then notification thread is easier (in case of dead handler).
 */

package com.wjholden.nmap;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager.NameNotFoundException;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.AutoCompleteTextView;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

public class MainActivity extends Activity {

	private transient Button btnStart, btnHelp, btnHistory, btnSubnet;
	private transient EditText etxtArguments;

	/**
	 * Changed from an EditText to an 
	 * <a href="http://developer.android.com/resources/tutorials/views/hello-autocomplete.html">AutoCompleteTextView</a>
	 * so that I can provide the user a nice list of historical targets, 
	 * pulled directly from the database with PipsDatabase.fetchTargets().
	 */
	private transient AutoCompleteTextView autoCompleteTarget;
	private transient TextView txtResults;

	public static ProgressDialog progressDialog;

	private static Context context;

	private transient boolean debug;
	private transient int currentVersion = 0;
	private transient boolean hasRoot;

	public static Handler handler;

	private transient Scan scan;

	private transient PipsDatabase db;

	/** Called when the activity is first created. */
	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.main);

		btnStart = (Button) findViewById(R.id.Start);
		btnHelp = (Button) findViewById(R.id.Help);
		btnHistory = (Button) findViewById(R.id.History);
		btnSubnet = (Button) findViewById(R.id.Subnet);

		autoCompleteTarget = (AutoCompleteTextView) findViewById(R.id.Target);
		etxtArguments = (EditText) findViewById(R.id.Arguments);

		txtResults = (TextView) findViewById(R.id.Results);

		context = this.getApplicationContext();
		
		btnStart.setEnabled(false); // must be enabled by license server
		PipsLicensing license = new PipsLicensing(context);
		license.start();
		
		btnStart.setOnClickListener(new View.OnClickListener() {

			public void onClick(View v) {
				if (scan == null || !scan.isAlive()) {
					scan = new Scan(Utilities.getApplicationFolder(context,
							"bin"), hasRoot, autoCompleteTarget.getText().toString(),
							etxtArguments.getText().toString(), 0, context);
					scan.start();
					btnStart.setEnabled(false);
				} else {
					scan.interrupt(); // TODO this does nothing
				}
			}
		});

		btnHelp.setOnClickListener(new View.OnClickListener() {

			public void onClick(View v) {
				if (scan == null || !scan.isAlive()) {
					scan = new Scan(Utilities.getApplicationFolder(context,
							"bin"), hasRoot, autoCompleteTarget.getText().toString(),
							"", 1, context);
					scan.start();
				} else {
					scan.interrupt(); // TODO this does nothing
				}
			}
		});
		
		btnHistory.setOnClickListener(new View.OnClickListener() {
			
			public void onClick(View v) {
				// TODO MAKE THIS REAL
				// right now it just shows the list activity.
				// needs to share scan result with other user.
				Intent intent = new Intent(MainActivity.this, PipsListActivity.class);
				startActivity(intent);
			}
		});
		
		btnSubnet.setOnClickListener(new View.OnClickListener() {
			
			public void onClick(View v) {
				Intent intent = new Intent(MainActivity.this, SubnetActivity.class);
				startActivity(intent);
			}
		});
	}

	public static Context getContext() {
		return (context);
	}

	protected void onPause() {
		db.close();

		final SharedPreferences.Editor preferencesEditor = getPreferences(0).edit();

		// wipe everything before adding anything
		preferencesEditor.clear();

		preferencesEditor.putString("target", autoCompleteTarget.getText().toString());
		preferencesEditor.putString("args", etxtArguments.getText().toString());
		preferencesEditor.putBoolean("DEBUG", debug);
		preferencesEditor.putInt("versionLastRun", currentVersion);

		preferencesEditor.commit(); // TODO once you get up to sdk 9 switch to apply().

		super.onPause();
	}

	@Override
	protected void onResume() {
		handler = new Handler() {

			@Override
			public void handleMessage(Message msg) {
				// TODO what happens if a thread is running when the handler is resumed?
				PipsError.log("Handler received: " + msg.what + " Object: "
						+ msg.obj);
				
				if (msg.obj != null && !(msg.obj instanceof String))
				{
					PipsError.log("Warning: handler received unexpected non-null Message.obj not instanceof String.");
				}

				switch (msg.what) {
				case Constants.INSTALL_COMPLETE:
					// TODO do nothing, no output to show user.
					break;
				case Constants.INSTALL_ERROR:
					// TODO show error to user.
					break;
				case Constants.PROGRESS_DIALOG_START:
					progressDialog = ProgressDialog.show(MainActivity.this, "",
							(String) msg.obj, true);
					break;
				case Constants.PROGRESS_DIALOG_DISMISS:
					progressDialog.dismiss();
					break;
				case Constants.PROGRESS_DIALOG_CHANGE_TEXT:
					if (progressDialog.isShowing())
						progressDialog.setMessage((String) msg.obj);
					else
						PipsError.log("Progress dialog is not showing but text changed.");
					break;
				case Constants.SCAN_ERROR_NULL_PROCESS:
					txtResults
							.setText("Unable to start compiled Nmap program.");
					btnStart.setEnabled(true);
					break;
				case Constants.SCAN_ERROR_STANDARD_ERROR:
					AlertDialog.Builder alert = new AlertDialog.Builder(
							MainActivity.this).setPositiveButton("OK",
							new DialogInterface.OnClickListener() {

								public void onClick(DialogInterface dialog,
										int which) {
									dialog.dismiss();
								}
							});
					alert.setMessage((String) msg.obj);
					alert.show();
					btnStart.setEnabled(true);
					break;
				case Constants.SCAN_COMPLETE:
					String result = (String) msg.obj;
					txtResults.setText(result);
					btnStart.setEnabled(true);
					
					bindAutoCompleteAdapter(); // rebind now that data is changed.
					break;
				case Constants.DB_SHOW_LAST_RESULT:
					String lastResult = db.fetchLastAsString();
					if (lastResult != null)
						txtResults.setText(lastResult);
					break;
				case Constants.LICENSE_ALLOW:
					btnStart.setEnabled(true);
					break;
				case Constants.LICENSE_FAIL:
				case Constants.LICENSE_ERROR:
					btnStart.setEnabled(false);
					AlertDialog.Builder alertLicensing = new AlertDialog.Builder(
							MainActivity.this).setPositiveButton("OK",
							new DialogInterface.OnClickListener() {

								public void onClick(DialogInterface dialog,
										int which) {
									dialog.dismiss();
								}
							});
					alertLicensing.setMessage("License verification failed. Contact developer at wjholden@gmail.com.");
					alertLicensing.show();
					break;
				default:
					PipsError.log("Handler received unexpected message that the switch statement does not allow.");
				}
			}
		};

		db = new PipsDatabase(this).open();
		handler.sendEmptyMessage(Constants.DB_SHOW_LAST_RESULT);
		
		bindAutoCompleteAdapter();

		SharedPreferences settings = getPreferences(0);
		debug = settings.getBoolean("DEBUG", false);
		PipsError.setDebug(debug);
		autoCompleteTarget.setText(settings.getString("target", ""));
		etxtArguments.setText(settings.getString("args", ""));
		int lastVersionRun = settings.getInt("versionLastRun", -1);
		try {
			currentVersion = getPackageManager().getPackageInfo(
					"com.wjholden.nmap", 0).versionCode;
		} catch (NameNotFoundException e) {
			PipsError.log(e.toString());
		} finally {
			if (currentVersion > lastVersionRun) {
				// TODO add progressdialog
				hasRoot = Utilities.canGetRoot();
				Install install = new Install(MainActivity.context,
						Utilities.getApplicationFolder(MainActivity.context,
								"bin"), hasRoot);
				install.start();
			}
		}
		// for initial testing TODO remove
		PipsError.setDebug(true);

		super.onResume();
	}
	
	/**
	 * Provides a nice list of historical targets for ease of use.
	 * I decided to create a method to do this since it needs to be called more than once -
	 * could not count on notifyDataSetChanged because the ArrayAdapter itself does not
	 * change with the database until you tell it to fetch updated information from said db.
	 */
	private void bindAutoCompleteAdapter()
	{
		ArrayAdapter<String> adapter = new ArrayAdapter<String>(this, R.layout.list_item, db.fetchTargets());
		autoCompleteTarget.setAdapter(adapter);
	}

}
