package com.wjholden.nmap;

import java.io.File;
import java.io.IOException;

import android.content.Context;
import android.content.pm.PackageManager.NameNotFoundException;

public final class Utilities {
	
	@SuppressWarnings("unused")
	private static final Utilities SINGLETON = new Utilities();
	
	private Utilities()
	{
		// singleton - does nothing.
	}

	public static String getApplicationFolder(final Context context, final String subfolder) {
		File appDir = null; 
		
		try {
			appDir = new File (getDataDirectory(context) + "/" + subfolder);
			
			// if it doesn't exist, create it
			if (appDir.exists())
			{
				PipsError.log(appDir.getAbsolutePath() + " exists.");
			}
			else
			{
				PipsError.log(appDir.getAbsolutePath() + " does not exist.");
				if (appDir.mkdirs())
				{
					PipsError.log("Created " + appDir.getAbsolutePath());
				}
				else
				{
					throw new Utilities.CannotCreateDirectoryException("Failed to create " + appDir.getAbsolutePath());
				}
			}
		}
		catch (CannotCreateDirectoryException e)
		{
			PipsError.log(e.toString());
			appDir = new File ("/tmp/");
		}
		
		return appDir.getAbsolutePath() + "/";
	}
	
	private static String getDataDirectory(final Context context)
	{
		String dataDirectory = "";
		try
		{
			dataDirectory = context.getPackageManager().getApplicationInfo("com.wjholden.nmap", 0).dataDir;
		}
		catch (NameNotFoundException e) {
			// this wouldn't be good, but it is very unlikely to happen.
			PipsError.log(e.toString());
		}
		return dataDirectory;
	}
	
	public static boolean canGetRoot()
	{
		// TODO
		return false;
	}
	
	/**
	 * Custom exception for handling errors where Runtime.getRuntime().exec(String) returned null. 
	 * @author William John Holden
	 * @version 1
	 */
	public static final class NullProcessException extends Exception {

		private static final long serialVersionUID = -6606740982523502676L;

		/**
		 * Use if Runtime.getRuntime().exec(String) returns null.
		 */
		public NullProcessException() {
			super();
		}
	}
	
	/**
	 * Custom exception extending IOException.
	 * This is useful because File.mkdirs() does not throw an IOException. 
	 * @author William John Holden
	 * @version 1
	 */
	public static final class CannotCreateDirectoryException extends IOException {

		private static final long serialVersionUID = -7566027000405830050L;

		/**
		 * If you find File.mkdirs() returns false, throw this exception.
		 * @param detailMessage Absolute path of the directory you attempted to create.
		 */
		public CannotCreateDirectoryException(final String detailMessage) {
			super(detailMessage);
		}
	}

	/**
	 * Custom exception for handling case where Nmap executable returned standard error.
	 * @author William John Holden
	 *
	 */
	public static final class StandardErrorNotEmptyException extends Exception {
		
		private static final long serialVersionUID = 866136689299038573L;

		/**
		 * Under normal circumstances there should be nothing on the standard error
		 * stream after running Nmap.  If there is, you need to throw this exception. 
		 * @param detailMessage Everything read from the standard error stream.
		 */
		public StandardErrorNotEmptyException (final String detailMessage) {
			super(detailMessage);
		}
	}
}
